@@ -1,15 +1,9 @@
 #!perl
 use strict;
 use diagnostics;
-use Config::AutoConf 0.310;
+use Config::AutoConf 0.311;
 use File::Temp qw/tempfile/;
 use Capture::Tiny qw/capture/;
-
-BEGIN {
-  no warnings 'redefine';
-  *Config::AutoConf::check_member = \&my_check_member;
-  *Config::AutoConf::check_members = \&my_check_members;
-}
 use POSIX qw/EXIT_SUCCESS/;
 use File::Spec;
 #
@@ -56,7 +50,6 @@ sub do_config_GNU {
     $ac->check_decl('isblank', { action_on_true => sub { $ac->define_var('HAVE_DECL_ISBLANK', 1) }, prologue => '#include <ctype.h>' });
     $ac->define_var('_REGEX_INCLUDE_LIMITS_H', 1);
     $ac->define_var('_REGEX_LARGE_OFFSETS', 1);
-    $ac->define_var('RE_ENABLE_I18N', 1);
     $ac->define_var('_PERL_I18N', 1);
     $ac->define_var('re_syntax_options', 'rpl_re_syntax_options');
     $ac->define_var('re_set_syntax', 'rpl_re_set_syntax');
@@ -273,101 +266,3 @@ int testrestrict() {
     $ac->define_var('restrict', $restrict);
   }
 }
-
-#
-# Until this is fixed in Config::AutoConf
-#
-sub my_check_member {
-    my $options = {};
-    scalar @_ > 2 and ref $_[-1] eq "HASH" and $options = pop @_;
-    my ( $self, $member ) = @_;
-    $self = $self->_get_instance();
-    defined($member)   or return croak("No type to check for");
-    ref($member) eq "" or return croak("No type to check for");
-
-    $member =~ m/^([^.]+)\.([^.]+)$/ or return croak("check_member(\"struct foo.member\", \%options)");
-    my $type = $1;
-    $member = $2;
-
-    my $cache_name = $self->_cache_type_name( "$type.$member" );
-    my $check_sub = sub {
-
-        my $body = <<ACEOF;
-  static $type check_aggr;
-  if( check_aggr.$member )
-    return 0;
-ACEOF
-        my $conftest = $self->lang_build_program( $options->{prologue}, $body );
-
-        my $have_member = $self->compile_if_else(
-            $conftest,
-            {
-                ( $options->{action_on_true}  ? ( action_on_true  => $options->{action_on_true} )  : () ),
-                ( $options->{action_on_false} ? ( action_on_false => $options->{action_on_false} ) : () )
-            }
-        );
-        $self->define_var(
-            Config::AutoConf::_have_member_define_name("$type.$member"),
-            $have_member ? $have_member : undef,
-            "defined when $type.$member is available"
-        );
-        $have_member;
-    };
-
-    $self->check_cached(
-        $cache_name,
-        "for $type.$member",
-        $check_sub,
-        {
-            ( $options->{action_on_cache_true}  ? ( action_on_true  => $options->{action_on_cache_true} )  : () ),
-            ( $options->{action_on_cache_false} ? ( action_on_false => $options->{action_on_cache_false} ) : () )
-        }
-    );
-};
-
-sub my_check_members {
-    my $options = {};
-    scalar @_ > 2 and ref $_[-1] eq "HASH" and $options = pop @_;
-    my ( $self, $members ) = @_;
-    $self = $self->_get_instance();
-
-    my %pass_options;
-    defined $options->{prologue}              and $pass_options{prologue}              = $options->{prologue};
-    defined $options->{action_on_cache_true}  and $pass_options{action_on_cache_true}  = $options->{action_on_cache_true};
-    defined $options->{action_on_cache_false} and $pass_options{action_on_cache_false} = $options->{action_on_cache_false};
-
-    my $have_members = 1;
-    foreach my $member (@$members)
-    {
-        $have_members &= !!(
-            $self->check_member(
-                $member,
-                {
-                    %pass_options,
-                    (
-                        $options->{action_on_member_true} && "CODE" eq ref $options->{action_on_member_true}
-                        ? ( action_on_true => sub { $options->{action_on_member_true}->($member) } )
-                        : ()
-                    ),
-                    (
-                        $options->{action_on_member_false} && "CODE" eq ref $options->{action_on_member_false}
-                        ? ( action_on_false => sub { $options->{action_on_member_false}->($member) } )
-                        : ()
-                    ),
-                }
-            )
-        );
-    }
-
-          $have_members
-      and $options->{action_on_true}
-      and ref $options->{action_on_true} eq "CODE"
-      and $options->{action_on_true}->();
-
-    $options->{action_on_false}
-      and ref $options->{action_on_false} eq "CODE"
-      and !$have_members
-      and $options->{action_on_false}->();
-
-    $have_members;
-};
