@@ -189,15 +189,15 @@ static void
 internal_function
 build_wcs_buffer (pTHX_ re_string_t *pstr)
 {
-#if (defined(_LIBC) || defined(_PERL_I18N))
-  unsigned char buf[MB_LEN_MAX];
-  assert (MB_LEN_MAX >= pstr->mb_cur_max);
-#else
-  unsigned char buf[64];
-#endif
   rpl__mbstate_t prev_st;
   Idx byte_idx, end_idx, remain_len;
   size_t mbclen;
+#if (defined(_LIBC) || defined(_PERL_I18N))
+  unsigned char buf[rpl__MB_LEN_MAX];
+  assert (rpl__MB_LEN_MAX >= pstr->mb_cur_max);
+#else
+  unsigned char buf[64];
+#endif
 
   /* Build the buffers from pstr->valid_len to either pstr->len or
      pstr->bufs_len.  */
@@ -263,9 +263,9 @@ build_wcs_upper_buffer (pTHX_ re_string_t *pstr)
   rpl__mbstate_t prev_st;
   Idx src_idx, byte_idx, end_idx, remain_len;
   size_t mbclen;
-#ifdef _LIBC
-  char buf[MB_LEN_MAX];
-  assert (MB_LEN_MAX >= pstr->mb_cur_max);
+#if (defined(_LIBC) || defined(_PERL_I18N))
+  char buf[rpl__MB_LEN_MAX];
+  assert (rpl__MB_LEN_MAX >= pstr->mb_cur_max);
 #else
   char buf[64];
 #endif
@@ -286,7 +286,7 @@ build_wcs_upper_buffer (pTHX_ re_string_t *pstr)
 	    {
 	      /* In case of a singlebyte character.  */
 	      pstr->mbs[byte_idx]
-		= toupper (pstr->raw_mbs[pstr->raw_mbs_idx + byte_idx]);
+		= rpl__toupper (pstr->raw_mbs[pstr->raw_mbs_idx + byte_idx]);
 	      /* The next step uses the assumption that rpl__wchar_t is encoded
 		 ASCII-safe: all ASCII values can be converted like this.  */
 	      pstr->wcs[byte_idx] = (rpl__wchar_t) pstr->mbs[byte_idx];
@@ -525,7 +525,7 @@ build_upper_buffer (pTHX_ re_string_t *pstr)
       if (BE (pstr->trans != NULL, 0))
 	ch = pstr->trans[ch];
       if (rpl__islower (ch))
-	pstr->mbs[char_idx] = toupper (ch);
+	pstr->mbs[char_idx] = rpl__toupper (ch);
       else
 	pstr->mbs[char_idx] = ch;
     }
@@ -701,9 +701,8 @@ re_string_reconstruct (pTHX_ re_string_t *pstr, Idx idx, int eflags)
 		  if (end < pstr->raw_mbs)
 		    end = pstr->raw_mbs;
 		  p = raw + offset - 1;
-#ifdef _LIBC
-		  /* We know the rpl__wchar_t encoding is UCS4, so for the simple
-		     case, ASCII characters, skip the conversion step.  */
+#if (defined(_LIBC) || defined(_PERL_I18N))
+		  /* For the simple case, ASCII characters, skip the conversion step.  */
 		  if (rpl__isascii (*p) && BE (pstr->trans == NULL, 1))
 		    {
 		      memset (&pstr->cur_state, '\0', sizeof (rpl__mbstate_t));
@@ -1534,14 +1533,14 @@ register_state (pTHX_ const re_dfa_t *dfa, re_dfastate_t *newstate,
 static void
 free_state (pTHX_ re_dfastate_t *state)
 {
-  re_node_set_free (aTHX_ &state->non_eps_nodes);
-  re_node_set_free (aTHX_ &state->inveclosure);
+  re_node_set_free (&state->non_eps_nodes);
+  re_node_set_free (&state->inveclosure);
   if (state->entrance_nodes != &state->nodes)
     {
-      re_node_set_free (aTHX_ state->entrance_nodes);
+      re_node_set_free (state->entrance_nodes);
       re_free (state->entrance_nodes);
     }
-  re_node_set_free (aTHX_ &state->nodes);
+  re_node_set_free (&state->nodes);
   re_free (state->word_trtable);
   re_free (state->trtable);
   re_free (state);
