@@ -129,10 +129,17 @@ typedef unsigned char bool;
 #else
 # define lock_define(name) SV *name;
 /* GNU regex expect lock_init(lock) to return 0 if success */
+/* Break on win32 ? */
+/*
 # define lock_init(lock) (SvSHARE(lock), 0)
-# define lock_fini(lock) /* There is no SvUNSHARE */
+# define lock_fini(lock)
 # define lock_lock(lock) SvLOCK(lock)
 # define lock_unlock(lock) SvUNLOCK(lock)
+*/
+# define lock_init(lock) 0
+# define lock_fini(lock)
+# define lock_lock(lock)
+# define lock_unlock(lock)
 #endif
 
 /* In case that the system doesn't have isblank().  */
@@ -193,6 +200,7 @@ typedef unsigned char bool;
 #undef rpl__wctype
 #undef rpl__isascii
 #undef rpl__isalnum
+#undef rpl__iswalnum
 #undef rpl__iscntrl
 #undef rpl__islower
 #undef rpl__isspace
@@ -224,6 +232,7 @@ typedef unsigned char bool;
 #   define rpl__wctype(c) wctype(c)
 #   define rpl__isascii(c) isascii(c)
 #   define rpl__isalnum(c) isalnum(c)
+#   define rpl__iswalnum(c) iswalnum(c)
 #   define rpl__iscntrl(c) iscntrl(c)
 #   define rpl__islower(c) islower(c)
 #   define rpl__isspace(c) isspace(c)
@@ -243,18 +252,21 @@ typedef unsigned char bool;
 #   define rpl__wcrtomb(s, wc, ps) wcrtomb(s, wc, ps)
 #   define rpl__towlower(wc) towlower(wc)
 #   define rpl__towupper(wc) towupper(wc)
+#   define rpl__tolower(wc) tolower(wc)
+#   define rpl__toupper(wc) toupper(wc)
 #   define rpl__mbstate_t mbstate_t 
 #   define rpl__MB_CUR_MAX MB_CUR_MAX
 #   define rpl__MB_LEN_MAX MB_LEN_MAX
 #   define rpl__WEOF WEOF
 # else
 #   define rpl__wint_t UV
-#   define rpl__wchar_t U8
+#   define rpl__wchar_t UV
 #   define rpl__mbsinit rpl_Perl_mbsinit
 #   define rpl__wctype_t rpl_Perl_wctype_t
 #   define rpl__wctype(property) rpl_Perl_wctype(aTHX_ property)
 #   define rpl__isascii(c) rpl_Perl_isascii(aTHX_ c)
 #   define rpl__isalnum(c) rpl_Perl_isalnum(aTHX_ c)
+#   define rpl__iswalnum(c) rpl_Perl_iswalnum(aTHX_ c)
 #   define rpl__iscntrl(c) rpl_Perl_iscntrl(aTHX_ c)
 #   define rpl__islower(c) rpl_Perl_islower(aTHX_ c)
 #   define rpl__isspace(c) rpl_Perl_isspace(aTHX_ c)
@@ -274,8 +286,10 @@ typedef unsigned char bool;
 #   define rpl__wcrtomb(s, wc, ps) rpl_Perl_wcrtomb(aTHX_ s, wc, ps)
 #   define rpl__towlower(wc) rpl_Perl_towlower(aTHX_ wc)
 #   define rpl__towupper(wc) rpl_Perl_towupper(aTHX_ wc)
+#   define rpl__tolower(wc) rpl_Perl_tolower(aTHX_ wc)
+#   define rpl__toupper(wc) rpl_Perl_toupper(aTHX_ wc)
 #   define rpl__mbstate_t rpl_Perl_mbstate_t 
-#   define rpl__MB_CUR_MAX rpl_Perl_MB_CUR_MAX()
+#   define rpl__MB_CUR_MAX rpl_Perl_MB_CUR_MAX(aTHX)
 #   define rpl__MB_LEN_MAX UTF8_MAXBYTES
 #   define rpl__WEOF ((UV)-1)
 typedef enum {
@@ -303,7 +317,7 @@ typedef struct {
 # define attribute_hidden
 #endif /* not _LIBC */
 
-#if (defined MB_CUR_MAX && HAVE_WCTYPE_H && HAVE_ISWCTYPE) || _LIBC
+#if (defined MB_CUR_MAX && HAVE_WCTYPE_H && HAVE_ISWCTYPE) || _LIBC || defined(_PERL_I18N)
 # define RE_ENABLE_I18N
 #endif
 
@@ -474,13 +488,21 @@ typedef struct
   rpl__wchar_t *mbchars;
 
   /* Collating symbols.  */
-# ifdef _LIBC
+# if (defined(_LIBC) || defined(_PERL_I18N))
+#   ifdef _LIBC
   int32_t *coll_syms;
+#   else
+  UV *coll_syms;
+#   endif
 # endif
 
   /* Equivalence classes. */
-# ifdef _LIBC
+# if (defined(_LIBC) || defined(_PERL_I18N))
+#   ifdef _LIBC
   int32_t *equiv_classes;
+#   else
+  UV *equiv_classes;
+#   endif
 # endif
 
   /* Range expressions. */
@@ -687,7 +709,7 @@ static unsigned int re_string_context_at (pTHX_ const re_string_t *input, Idx id
 
 /*  free(p) <==> Safefree(p) */
 /* #define re_free(p) free (p) */
-#define re_free(p) Safefree (p)
+#define re_free(p) do { if (p != NULL) { Safefree (p); } p = NULL; } while (0)
 
 struct bin_tree_t
 {
@@ -728,7 +750,7 @@ typedef struct bin_tree_storage_t bin_tree_storage_t;
 
 #define IS_WORD_CHAR(ch) (rpl__isalnum (ch) || (ch) == '_')
 #define IS_NEWLINE(ch) ((ch) == NEWLINE_CHAR)
-#define IS_WIDE_WORD_CHAR(ch) (iswalnum (ch) || (ch) == L'_')
+#define IS_WIDE_WORD_CHAR(ch) (rpl__iswalnum (ch) || (ch) == L'_')
 #define IS_WIDE_NEWLINE(ch) ((ch) == WIDE_NEWLINE_CHAR)
 
 #define NOT_SATISFY_PREV_CONSTRAINT(constraint,context) \
@@ -838,6 +860,9 @@ typedef struct
   Idx nsub_tops;
   Idx asub_tops;
   re_sub_match_top_t **sub_tops;
+#ifdef _PERL_I18N
+  SV *sv;
+#endif
 } re_match_context_t;
 
 typedef struct
@@ -1037,7 +1062,7 @@ re_string_wchar_at (pTHX_ const re_string_t *pstr, Idx idx)
 
 #ifndef _LIBC
 #ifdef _PERL_I18N
-size_t rpl_Perl_MB_CUR_MAX() {
+size_t rpl_Perl_MB_CUR_MAX(pTHX) {
   size_t rc;
 
   rc = rpl__MB_LEN_MAX;
@@ -1082,6 +1107,23 @@ int rpl_Perl_isalnum(pTHX_ UV c) {
   return rc;
 }
 
+int rpl_Perl_iswalnum(pTHX_ UV c) {
+  int rc;
+#ifndef isALNUM_LC_uvchr
+#define isALNUM_LC_uvchr isALNUM_LC
+#ifndef isALNUM_LC
+#define isALNUM_LC isALNUM
+#endif
+#endif
+  rc = isALNUM_LC_uvchr(c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_iswalnum(%d) ==> %d\n", (int) c, rc);
+#endif
+
+  return rc;
+}
+
 int rpl_Perl_iscntrl(pTHX_ UV c) {
   int rc;
 #ifndef isCNTRL_LC_uvchr
@@ -1280,12 +1322,12 @@ int rpl_Perl_iswlower(pTHX_ UV wc) {
 }
 
 /* Our mb implementations are all stateless */
-size_t rpl_Perl_mbrtowc(pTHX_ U8 *restrict pwc, const char *restrict s, size_t n, void *restrict ps) {
+size_t rpl_Perl_mbrtowc(pTHX_ UV *restrict pwc, const char *restrict s, size_t n, void *restrict ps) {
   STRLEN ch_len;
   UV     ord;
   size_t rc;
 #ifndef NDEBUG
-  void octdump(void *mem, unsigned int len);
+  void octdump(pTHX_ const void *mem, unsigned int len);
 #endif
 
   if (s == NULL) {
@@ -1295,7 +1337,7 @@ size_t rpl_Perl_mbrtowc(pTHX_ U8 *restrict pwc, const char *restrict s, size_t n
   }
 
 #ifndef NDEBUG
-  octdump(s, n);
+  octdump(aTHX_ s, n);
 #endif
 
   if (n == 0) {
@@ -1305,7 +1347,7 @@ size_t rpl_Perl_mbrtowc(pTHX_ U8 *restrict pwc, const char *restrict s, size_t n
     ord = utf8n_to_uvchr((U8 *) s, n, &ch_len, UTF8_CHECK_ONLY);
     if (ord > 0 || *s == 0) {
       if (pwc != NULL) {
-        Copy (s, pwc, ch_len, U8);
+        *pwc = ord;
       }
       rc = (ord == 0) ? 0 : ch_len;
     } else {
@@ -1323,7 +1365,7 @@ size_t rpl_Perl_mbrtowc(pTHX_ U8 *restrict pwc, const char *restrict s, size_t n
   return rc;
 }
 
-int rpl_Perl_mbtowc(pTHX_ rpl__wchar_t *restrict pwc, const char *restrict s, size_t n) {
+int rpl_Perl_mbtowc(pTHX_ UV *restrict pwc, const char *restrict s, size_t n) {
   int rc;
 
   static rpl_Perl_mbstate_t state;
@@ -1362,7 +1404,7 @@ rpl__wint_t rpl_Perl_btowc (pTHX_ int c) {
 
     buf[0] = (U8)c;
     if (rpl_Perl_mbtowc(aTHX_ &wc, buf, 1) >= 0) {
-      rc = wc;
+      rc = (rpl__wint_t) wc;
     }
   }
 
@@ -1373,7 +1415,7 @@ rpl__wint_t rpl_Perl_btowc (pTHX_ int c) {
   return rc;
 }
 
-int rpl_Perl_iswctype(pTHX_ rpl__wint_t wi, wctype_t wt) {
+int rpl_Perl_iswctype(pTHX_ rpl__wint_t wi, rpl__wctype_t wt) {
   int rc;
 
   if (wi == rpl__WEOF) {
@@ -1482,7 +1524,6 @@ int Perl_wctomb(pTHX_ char *restrict s, rpl__wchar_t wc) {
   bool   is_utf8 = 1;
   U8    *bytes;
   STRLEN len;
-  char *p;
 
   if (s == NULL) {
     return 0;
@@ -1534,12 +1575,12 @@ size_t rpl_Perl_wcrtomb (pTHX_ char *s, rpl__wchar_t wc, rpl__mbstate_t *ps) {
     }
 }
 
-rpl__wint_t rpl_Perl_towlower(pTHX_ rpl__wint_t wc) {
+rpl__wint_t rpl_Perl_towlower(pTHX_ UV wc) {
   U8     s[UTF8_MAXBYTES_CASE+1];
   rpl__wint_t rc;
   STRLEN len;
 
-  rc = toLOWER_uni((UV) wc, s, &len);
+  rc = toLOWER_uni(wc, s, &len);
 
 #ifndef NDEBUG
   fprintf(stderr, "rpl_Perl_towlower(%d) ==> %d\n", (int) wc, (int) rc);
@@ -1549,12 +1590,12 @@ rpl__wint_t rpl_Perl_towlower(pTHX_ rpl__wint_t wc) {
 
 }
 
-rpl__wint_t rpl_Perl_towupper(pTHX_ rpl__wint_t wc) {
+rpl__wint_t rpl_Perl_towupper(pTHX_ UV wc) {
   U8     s[UTF8_MAXBYTES_CASE+1];
   rpl__wint_t rc;
   STRLEN len;
 
-  rc = toUPPER_uni((UV) wc, s, &len);
+  rc = toUPPER_uni(wc, s, &len);
 
 #ifndef NDEBUG
   fprintf(stderr, "rpl_Perl_towlower(%d) ==> %d\n", (int) wc, (int) rc);
@@ -1564,13 +1605,41 @@ rpl__wint_t rpl_Perl_towupper(pTHX_ rpl__wint_t wc) {
 
 }
 
+rpl__wint_t rpl_Perl_tolower(pTHX_ UV c) {
+  rpl__wint_t rc;
+
+  /* The caller made sure that it fits in 8 bytes */
+  rc = toLOWER((U8) c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_tolower(%d) ==> %d\n", (int) c, (int) rc);
+#endif
+
+  return rc;
+
+}
+
+rpl__wint_t rpl_Perl_toupper(pTHX_ UV c) {
+  rpl__wint_t rc;
+
+  /* The caller made sure that it fits in 8 bytes */
+  rc = toUPPER((U8) c);
+
+#ifndef NDEBUG
+  fprintf(stderr, "rpl_Perl_toupper(%d) ==> %d\n", (int) c, (int) rc);
+#endif
+
+  return rc;
+
+}
+
 #endif /* _PERL_I18N */
 #endif /* _LIBC */
 
 # ifndef NOT_IN_libc
 static int
 internal_function __attribute__ ((pure, unused))
-re_string_elem_size_at (pTHX_ const re_string_t *pstr, Idx idx)
+re_string_elem_size_at (pTHX_ const re_string_t *pstr, SV *sv, Idx idx)
 {
 #  ifdef _LIBC
   const unsigned char *p, *extra;
@@ -1593,22 +1662,30 @@ re_string_elem_size_at (pTHX_ const re_string_t *pstr, Idx idx)
 #  else
 #    ifdef _PERL_I18N
     {
-      STRLEN len = pstr->len - idx;
-      U8* utf8 = bytes_to_utf8((U8 *)(pstr->mbs + idx), &len);
-      SV *sv_tmp = newSVpvn_utf8(utf8, len, 1);
-      I32 startOffset = 0;
-      I32 length = 1;
+      if (! DO_UTF8(sv)) {
+        /* Per def perl's non UTF-8 is one byte */
+#ifndef NDEBUG
+        fprintf(stderr, "re_string_elem_size_at(.., Idx=%d) => 1\n", (int) idx);
+#endif
+        return 1;
+      } else {
+        /* We know that pstr->mbs is at offset raw_mbs_idx v.s. original string */
+        I32 offset = pstr->raw_mbs_idx + idx;
+        I32 len = 1;
 
-      sv_pos_u2b(sv_tmp, &startOffset, &length);
-
-      Safefree(utf8);
-      SvREFCNT_dec(sv_tmp);
-
-      return length;
+        sv_pos_b2u(sv, &offset);
+        sv_pos_u2b(sv, &offset, &len);
+#ifndef NDEBUG
+        fprintf(stderr, "re_string_elem_size_at(.., Idx=%d) => %d\n", (int) idx, (int) len);
+#endif
+        return (int) len;
+      }
+      
     }
-#    endif
-#  endif /* _LIBC */
+#    else
     return 1;
+#    endif  
+#  endif /* _LIBC */
 }
 # endif
 #endif /* RE_ENABLE_I18N */
@@ -1638,7 +1715,7 @@ re_string_elem_size_at (pTHX_ const re_string_t *pstr, Idx idx)
 #define OCTDUMP_COLS 8
 #endif
  
-void octdump(void *mem, unsigned int len)
+void octdump(pTHX_ const void *mem, unsigned int len)
 {
   unsigned int i, j;
         
